/*
 * Remote Procedure Call Server
 *
 *  Created on: Jan 30 2015
 *      Author: bdugan
 */
#include <ctime>
#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>
#include <Weather.h>

using namespace std;

int main() {
  try {
    ////////////////////////////////////////////////////////////////////
    // Create a socket.  
    ////////////////////////////////////////////////////////////////////
    int s = socket(PF_INET,SOCK_STREAM,0);

    if (s <= 0) {
      cerr << "server: Socket creation failed." << endl;
      exit(-1);
    }

    ////////////////////////////////////////////////////////////////////
    // Create a port to listen to for client connections
    ////////////////////////////////////////////////////////////////////
    sockaddr_in address;
    int addressSize = sizeof(sockaddr);
    address.sin_family=AF_INET;
    address.sin_addr.s_addr = INADDR_ANY;
    address.sin_port  = htons(15000); // <---------------- port is 15000

    int status=0;
    status = bind(s,(struct sockaddr*) &address, addressSize);

    if (status != 0) {
      cerr << "server: Bind failed, unable to create port." << endl;
      exit(-1);
    }

    ////////////////////////////////////////////////////////////////////
    // Wait for client to connect to port
    ////////////////////////////////////////////////////////////////////
    cout << "server: Waiting for client to connect" << endl;
    listen(s,3);
    int cs = accept(s,(struct sockaddr*) &address,(socklen_t *) &addressSize);

    if (cs <= 0) {
      cerr << "server: Accept failed, client can't connect to port." << endl;
      cout << strerror(cs) << endl;
      exit(-1);
    }

    // We now have TWO sockets:
    //   s  - used to listen for connections from a receiver
    //   cs - used to send/recv messages between sender and receiver
    //        this socket was actually created by the receiver.
    //
    cout << "Server and client have established communication." << endl;

    int bytesReceived;
    #define ACKLENGTH 4
    #define BYTEARRAYLENGTH 10000
    unsigned char byteArray[BYTEARRAYLENGTH];
    unsigned char ack[ACKLENGTH];

    // Get a weather object.
    Weather weather;

    while (true) {
      int requestType;

      bytesReceived = recv(cs,byteArray,BYTEARRAYLENGTH,0);
      ((unsigned char *) &requestType)[0] = byteArray[0];
      ((unsigned char *) &requestType)[1] = byteArray[1];
      ((unsigned char *) &requestType)[2] = byteArray[2];
      ((unsigned char *) &requestType)[3] = byteArray[3];

      switch (requestType) {
      case 0:
	send(cs,&weather.temperature, sizeof(weather.temperature),0);
	break;
	
      case 1:
	send(cs,weather.getSky().c_str(),weather.getSky().length(),0);
	break;
	
      case 2:
	send(cs,&weather.humidity, sizeof(weather.humidity),0);
	break;
	
      default:
	cout << "RPCServer: Error, unsupported request type" << endl;
	break;
      }
    }
  } catch (std::exception& e) {
    std::cerr << "main: " << e.what() << std::endl;
  }
}
